#include "searchtimer_thread.h"
#include "epgsearchcfg.h"
#include "epgsearchcats.h"
#include "tools.h"
#include "changrp.h"
#include "recdone.h"
#include <vdr/tools.h>
#include <vdr/plugin.h>

#if VDRVERSNUM >= 10330 || defined(PATCH_CUSTOMSERVICES)
#include "services.h"
#endif

extern int updateForced;

cSearchTimerThread *cSearchTimerThread::m_Instance = NULL;

cSearchTimerThread::cSearchTimerThread(void)
: cThread("EPGSearch: searchtimer")
{
    m_Active = false;
    m_lastUpdate = time(NULL);
}

cSearchTimerThread::~cSearchTimerThread() {
    if (m_Active) 
	Stop();
}

void cSearchTimerThread::Init(void) {
    if (!EPGSearchConfig.useSearchTimers)
	return;
    if (m_Instance == NULL) {
	m_Instance = new cSearchTimerThread;
	m_Instance->Start();
    }
}

void cSearchTimerThread::Exit(void) {
    if (m_Instance != NULL) {
	m_Instance->Stop();
	DELETENULL(m_Instance);
    }
}

void cSearchTimerThread::Stop(void) {
    m_Active = false;
    Cancel(6);
}

cTimer *cSearchTimerThread::GetTimer(cSearchExt *searchExt, const cEvent *pEvent, bool& bTimesMatchExactly)
{
    cChannel *channel = Channels.GetByChannelID(pEvent->ChannelID(), true);
    if (!channel)
	return NULL;

    bool UseVPS = searchExt->useVPS && pEvent->Vps() && Setup.UseVps;
    struct tm tm_r;
    time_t eStart;
    if (!UseVPS)
	eStart = pEvent->StartTime();
    else
	eStart = pEvent->Vps();
    time_t eStop;
    if (!UseVPS)
	eStop = pEvent->EndTime();
    else
	eStop = pEvent->Vps() + pEvent->Duration();

    tm *tmStartEv = localtime_r(&eStart, &tm_r);
    
    for (cTimer *ti = Timers.First(); ti; ti = Timers.Next(ti)) 
    {
	if (ti->Channel() != channel)
	    continue;

	time_t tStart = ti->StartTime();
	time_t tStop = ti->StopTime();
	if (!UseVPS)
	{
	    tStart += searchExt->MarginStart * 60;
	    tStop  -= searchExt->MarginStop * 60; 
	}

	tm *tmStartTi = localtime_r(&tStart, &tm_r);
	if (tmStartEv->tm_mday != tmStartTi->tm_mday)
	    continue;

	bTimesMatchExactly = (tStart == eStart && tStop == eStop);
	
	if (abs(tStart - eStart) < 10 * 60 && abs(tStop - eStop) < 10 * 60) // accept a difference of max 10 min.
	    return ti;
    }
    return NULL;
}

void cSearchTimerThread::Action(void) 
{
    if (EPGSearchConfig.useExternalSVDRP && !SVDRPSendCmd)
    {
	esyslog("EPGSearch: ERROR - SVDRPSend script not specified or does not exist (use -f option)");
	return;
    }
    m_Active = true;
    // let VDR do its startup
    sleepSec(20);
    time_t nextUpdate = time(NULL);
    while (m_Active) 
    {
	time_t now = time(NULL);
	if (now >= nextUpdate || NeedUpdate())
	{
	    if (Timers.BeingEdited()) 		
	    {
		sleepSec(1);
		continue;
	    }
	    isyslog("EPGSearch: search timer update started");
	    LogFile.Log(1,"search timer update started");

	    cSearchExt *searchExt = SearchExts.First();
	    while (searchExt) 
	    {
		if (!searchExt->useAsSearchTimer)
		{
		    searchExt = SearchExts.Next(searchExt);
		    continue;
		}
		sleepMSec(1);
		cSearchResults* pSearchResults = searchExt->Run(false, true);
		if (!pSearchResults)
		{
		    searchExt = SearchExts.Next(searchExt);
		    continue;
		}
		pSearchResults->SortBy(CompareEventTime);

		if (searchExt->avoidRepeats)
		    searchExt->CheckRepeatTimers(pSearchResults);

		for (cSearchResult* pResultObj = pSearchResults->First(); 
		     pResultObj; 
		     pResultObj = pSearchResults->Next(pResultObj))
		{
			const cEvent* pEvent = pResultObj->event;
			if (!pEvent)
			    continue;
			    
			cChannel *channel = Channels.GetByChannelID(pEvent->ChannelID(), true);
			if (!channel)
			    continue;

			int index = 0;
			cTimer *timer = new cTimer(pEvent);

			// create the file 
			char* file = NULL;
			if ((file = searchExt->BuildFile(pEvent)) != NULL)
			{
			    timer->SetFile(file);
			    free(file);
			}
			int Priority = searchExt->Priority;
			int Lifetime = searchExt->Lifetime;
			
			// search for an already existing timer
			bool bTimesMatchExactly = false;
			cTimer *t = GetTimer(searchExt, pEvent, bTimesMatchExactly);
			if (t) 
			{ // already exists
			    if (!t->HasFlags(tfActive))
			    { // do not update inactive timers
				LogFile.Log(2,"timer for '%s~%s' (%s - %s, channel %d) not active - won't be touched", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
				delete timer;
				continue;
			    }
			    
			    if (!pResultObj->needsTimer && !t->Recording()) // not needed
			    {
				LogFile.Log(1,"delete timer for '%s~%s' (%s - %s, channel %d)'", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
				DelTimer(t->Index()+1);
				delete timer;
				continue;
			    }
			    
			    char* pFile = NULL; // File is prepared for svdrp, so prepare t->File for comparision too
			    asprintf(&pFile, "%s", t->File());
			    pFile = strreplace(pFile, ':', '|');
			    pFile = strreplace(pFile, " ~", "~");
			    pFile = strreplace(pFile, "~ ", "~");
			    
			    char* Summary =  SummaryExtended(searchExt, t, pEvent);
			    
			    if (bTimesMatchExactly && strcmp(pFile, timer->File()) == 0 && 
				(t->Summary() != NULL && strcmp(t->Summary(), Summary) == 0))
			    { // dir, title, episode name and summary have not changed
				free(Summary);
				delete timer;
				free(pFile);
				continue;
			    }
			    else
			    {
				index = t->Index()+1;
				Priority = t->Priority();
				Lifetime = t->Lifetime();
			    }
			    free(pFile);
			    free(Summary);
			}			       
			else
			{
			    if (!pResultObj->needsTimer)
			    {
				delete timer;
				continue;
			    }
			}
		
			if (index == 0 && searchExt->announceOnly) // only announce if there is no timer for the event
			{
			    char* Message = NULL;
			    asprintf(&Message, "%s: %s - %s %s", pEvent->Title(), CHANNELNAME(channel), GETDATESTRING(pEvent), GETTIMESTRING(pEvent));
			    SendMsg(Message);
			    free(Message);
			    sleepSec(5);
			    delete timer;
			    continue;
			}
			
			AddModTimer(timer, index, searchExt, pEvent, Priority, Lifetime);
			delete timer;
			if (index == 0)
			    LogFile.Log(1,"added timer for '%s~%s' (%s - %s); search timer: '%s'", pEvent->Title(), pEvent->ShortText()?pEvent->ShortText():"", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), searchExt->search);
			else
			    LogFile.Log(1,"modified timer %d for '%s~%s' (%s - %s); search timer: '%s'", index, pEvent->Title(), pEvent->ShortText()?pEvent->ShortText():"", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), searchExt->search);
		}
		searchExt = SearchExts.Next(searchExt);
	    }
	    m_lastUpdate = time(NULL);
	    nextUpdate = m_lastUpdate + (EPGSearchConfig.UpdateIntervall * 60);
	    isyslog("EPGSearch: search timer update finished");
	    LogFile.Log(1,"search timer update finished");

	    // check for conflicts
	    if (EPGSearchConfig.checkTimerConflicts)
	    {
		isyslog("EPGSearch: check for timer conflicts");
		LogFile.Log(1,"check for timer conflicts");
		cPlugin *p = cPluginManager::GetPlugin("timeline");
		if (p)
		{
		    if (!Timers.BeingEdited()) 		
		    {
			char *args[]={"timeline_command_interface","conflicts"};
			bool timeline_found_conflict=p->ProcessArgs(1,args);
			if (timeline_found_conflict)
			{
			    isyslog("EPGSearch: conflict found");	    
			    LogFile.Log(1,"conflict found");	    
			    SendMsg(tr("Attention: conflicting timers found!"));			
			}
		    }
		}
		isyslog("EPGSearch: check for timer conflicts - done");	    
		LogFile.Log(1,"check for timer conflicts - done");	    
	    }

	    // delete expired recordings
	    CheckExpiredRecs();
	    LogFile.Log(1, "check for expired recordings done");
	    if (updateForced == 2)
	    {
		char *cmdbuf = NULL;    
		asprintf(&cmdbuf, "MESG %s", tr("Search timer update done!"));    
		SendViaSVDRP(cmdbuf);		
		free(cmdbuf); 
	    }

	    // reset service call flag
	    updateForced = 0;
	}
	sleepSec(5);
    };
    isyslog("EPGSearch: Leaving search timer thread");
}

bool cSearchTimerThread::NeedUpdate()
{
    return  (m_lastUpdate <= LastModifiedTime(AddDirectory(CONFIGDIR, ".epgsearchupdate")) || updateForced>0);
}

int cSearchTimerThread::GetCountTimers(const cEvent* event, bool compareTitle, bool compareSubtitle, bool compareSummary, unsigned long catvaluesAvoidRepeat, cTimer** first)
{
    if (!event) return 0;

    *first = NULL;
    int count = 0;
    for (cTimer *ti = Timers.First(); ti; ti = Timers.Next(ti)) 
    {
	if (!ti->HasFlags(tfActive))
	    continue;

	const cEvent* t_event = ti->Event();
	if (!t_event)
	    t_event = GetEvent(ti);
	if (!t_event)
	    continue;

	if (EventsMatch(event, t_event, compareTitle, compareSubtitle, compareSummary, catvaluesAvoidRepeat))
	{
	    if (!*first) *first = ti;
	    else
		if ((*first)->StartTime() > ti->StartTime())
		    *first = ti;	    
	    count++;
	}
    }
    return count;
}

char* cSearchTimerThread::SummaryExtended(cSearchExt* searchExt, cTimer* Timer, const cEvent* pEvent)
{
    char* addSummaryHeader = NULL;
    const char* title = pEvent->Title();
    const char* subtitle = pEvent->ShortText();

    asprintf(&addSummaryHeader, "%s\n%s%s%s", 
	     title?title:"", 
	     subtitle?tr("Subtitle"):"", subtitle?": ":"", subtitle?subtitle:"");

    char* addSummaryFooter = NULL;
    asprintf(&addSummaryFooter, "%s: %d - %s\n%s: %s\n%s: %d", 
	     tr("Channel"), Timer->Channel()->Number(), CHANNELNAME(Timer->Channel()),
	     tr("Search timer"), searchExt->search,
	     "S-ID", searchExt->ID);

    // already there?
    if (pEvent->Description() && strstr(pEvent->Description(), addSummaryHeader) && 
	strstr(pEvent->Description(), addSummaryFooter))
    {
	free(addSummaryHeader);
	free(addSummaryFooter);
	return strdup(pEvent->Description());
    }
    else
    {
	char* tmpSummary = NULL;
	asprintf(&tmpSummary, "%s\n\n%s\n%s", 
		 addSummaryHeader,
		 pEvent->Description()?pEvent->Description():"", 
		 addSummaryFooter);
	free(addSummaryHeader);
	free(addSummaryFooter);
	return tmpSummary;
    }
}

void cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searchExt, const cEvent* pEvent, int Prio, int Lifetime) 
{
    char *cmdbuf = NULL;

    static char bufStart[25];
    static char bufEnd[25];

    struct tm tm_r;    time_t eStart = pEvent->StartTime();
    time_t eStop = pEvent->EndTime();
    time_t start = eStart - (searchExt->MarginStart * 60);
    time_t stop  = eStop + (searchExt->MarginStop * 60);
    int Flags = Timer->Flags();
    if (searchExt->useVPS && pEvent->Vps() && Setup.UseVps)
    {
	start = pEvent->Vps();
	stop = start + pEvent->Duration();
    }
    else
	Flags = 1; // don't use VPS, if not set in this search

    strftime(bufStart, sizeof(bufStart), "%H%M", localtime_r(&start, &tm_r));
    strftime(bufEnd, sizeof(bufEnd), "%H%M", localtime_r(&stop, &tm_r));

    // add additonal info
    char* tmpSummary = SummaryExtended(searchExt, Timer, pEvent);
    strreplace(tmpSummary, '\n', '|');

    if (index==0)
#if VDRVERSNUM && VDRVERSNUM >= 10323
	asprintf(&cmdbuf, "NEWT %d:%d:%s:%s:%s:%d:%d:%s:%s", 
#else
	asprintf(&cmdbuf, "NEWT %d:%d:%d:%s:%s:%d:%d:%s:%s", 
#endif
		 Flags,
		 Timer->Channel()->Number(),
#if VDRVERSNUM && VDRVERSNUM >= 10323
		 *Timer->PrintDay(start, Timer->WeekDays()),
#else
		 localtime_r(&start, &tm_r)->tm_mday,
#endif
		 bufStart,
		 bufEnd,
		 Prio, 
		 Lifetime, 
		 Timer->File(),
		 tmpSummary);
		 else
#if VDRVERSNUM && VDRVERSNUM >= 10323
	asprintf(&cmdbuf, "MODT %d %d:%d:%s:%s:%s:%d:%d:%s:%s", 
#else
	asprintf(&cmdbuf, "MODT %d %d:%d:%d:%s:%s:%d:%d:%s:%s", 
#endif
		 index,
		 Flags,
		 Timer->Channel()->Number(),
#if VDRVERSNUM && VDRVERSNUM >= 10323
		 *Timer->PrintDay(start, Timer->WeekDays()),
#else
		 localtime_r(&start, &tm_r)->tm_mday,
#endif
		 bufStart,
		 bufEnd,
		 Prio, 
		 Lifetime, 
		 Timer->File(),
		 tmpSummary);

    SendViaSVDRP(cmdbuf);
	      
    free(cmdbuf);
    free(tmpSummary);
}

void cSearchTimerThread::DelTimer(int index)
{
    char *cmdbuf = NULL;    
    asprintf(&cmdbuf, "DELT %d", index);    
    LogFile.Log(2, "delete timer %d", index);
    SendViaSVDRP(cmdbuf);
	      
    free(cmdbuf); 
}

void cSearchTimerThread::DelRecording(int index)
{
    char *cmdbuf = NULL;   // first LSTR required 
    asprintf(&cmdbuf, "DELR %d", index);    
    LogFile.Log(2, "delete recording %d", index);
    SendViaSVDRP(cmdbuf);
	      
    free(cmdbuf); 
}

void cSearchTimerThread::CheckExpiredRecs()
{
#if VDRVERSNUM <= 10310    
    cRecordings Recordings;
    bool recordings = Recordings.Load();
    if (!recordings)
	return;
#endif

#if VDRVERSNUM >= 10333    
    cThreadLock RecordingsLock(&Recordings);
#endif
    cList<cRecordingObj> DelRecordings;
    for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording)) 
    {
	LogFile.Log(3, "check recording %s from %s for expiration", recording->Name(), DAYDATETIME(recording->start));
#if VDRVERSNUM <= 10324 
	const char* summary = recording->Summary();
#else
	if (!recording->Info()) continue;
	const char* summary = recording->Info()->Description();
#endif
	if (isempty(summary)) continue;
	
	char* searchID = GetExtEPGValue(summary, "S-ID");	
	char* searchName = GetExtEPGValue(summary, tr("Search timer"));	
	if (!searchID || !searchName) continue;
	cSearchExt* search = SearchExts.GetSearchFromID(atoi(searchID));
	if (!search || strcmp(search->search, searchName) != 0) continue;
	LogFile.Log(3, "recording triggered from search timer %s", search->search);
	if (search->delAfterDays == 0) continue;
	time_t now = time(NULL);
	int daysBetween = int(double((now - recording->start)) / (60*60*24));
	if (daysBetween  >= search->delAfterDays)
	    DelRecordings.Add(new cRecordingObj(recording, search));
	else
	    LogFile.Log(3, "recording will expire in %d days, search timer %s", search->delAfterDays - daysBetween, search->search);
    }    
    for (cRecordingObj *recordingObj = DelRecordings.First(); recordingObj; recordingObj = DelRecordings.Next(recordingObj)) 
    {
	cRecording* recording = recordingObj->recording;
	LogFile.Log(1, "delete expired recording %s from %s, search timer %s", recording->Name(), DAYDATETIME(recording->start), recordingObj->search->search);
	cRecordControl *rc = cRecordControls::GetRecordControl(recording->FileName());
	if (!rc) 
	{
	    if (!recording->Delete())
		LogFile.Log(1, "error deleting recording!");
	    else
#if VDRVERSNUM >= 10312    
		::Recordings.DelByName(recording->FileName());
#else
	    Recordings.Del(recording);
#endif
	}
	else
	    LogFile.Log(1, "recording already in use by a timer!");
    }
}
